@@ -1,7 +1,8 @@
 from __future__ import annotations
 
 from collections import defaultdict
-from typing import Iterator, Sequence, cast, overload
+from collections.abc import Iterator, Sequence
+from typing import TYPE_CHECKING, cast, overload
 
 from cognite.client._api_client import APIClient
 from cognite.client._constants import DATA_MODELING_DEFAULT_LIMIT_READ
@@ -11,13 +12,22 @@ from cognite.client.data_classes.data_modeling.ids import (
     _load_identifier,
 )
 from cognite.client.data_classes.data_modeling.views import View, ViewApply, ViewFilter, ViewList
+from cognite.client.utils._concurrency import ConcurrencySettings
 
-from ._data_modeling_executor import get_data_modeling_executor
+if TYPE_CHECKING:
+    from cognite.client import CogniteClient
+    from cognite.client.config import ClientConfig
 
 
 class ViewsAPI(APIClient):
     _RESOURCE_PATH = "/models/views"
 
+    def __init__(self, config: ClientConfig, api_version: str | None, cognite_client: CogniteClient) -> None:
+        super().__init__(config, api_version, cognite_client)
+        self._DELETE_LIMIT = 100
+        self._RETRIEVE_LIMIT = 100
+        self._CREATE_LIMIT = 100
+
     @overload
     def __call__(
         self,
@@ -27,8 +37,7 @@ class ViewsAPI(APIClient):
         include_inherited_properties: bool = True,
         all_versions: bool = False,
         include_global: bool = False,
-    ) -> Iterator[View]:
-        ...
+    ) -> Iterator[View]: ...
 
     @overload
     def __call__(
@@ -39,8 +48,7 @@ class ViewsAPI(APIClient):
         include_inherited_properties: bool = True,
         all_versions: bool = False,
         include_global: bool = False,
-    ) -> Iterator[ViewList]:
-        ...
+    ) -> Iterator[ViewList]: ...
 
     def __call__(
         self,
@@ -101,7 +109,10 @@ class ViewsAPI(APIClient):
         """`Retrieve a single view by id. <https://developer.cognite.com/api#tag/Views/operation/byExternalIdsViews>`_
 
         Args:
-            ids (ViewIdentifier | Sequence[ViewIdentifier]): View identifier(s)
+            ids (ViewIdentifier | Sequence[ViewIdentifier]): The view identifier(s). This can be given as a tuple of
+                strings or a ViewId object. For example, ("my_space", "my_view"), ("my_space", "my_view", "my_version"),
+                or ViewId("my_space", "my_view", "my_version"). Note that version is optional, if not provided, all versions
+                will be returned.
             include_inherited_properties (bool): Whether to include properties inherited from views this view implements.
             all_versions (bool): Whether to return all versions. If false, only the newest version is returned,
 
@@ -111,8 +122,8 @@ class ViewsAPI(APIClient):
         Examples:
 
                 >>> from cognite.client import CogniteClient
-                >>> c = CogniteClient()
-                >>> res = c.data_modeling.views.retrieve(('mySpace', 'myView', 'v1'))
+                >>> client = CogniteClient()
+                >>> res = client.data_modeling.views.retrieve(('mySpace', 'myView', 'v1'))
 
         """
         identifier = _load_identifier(ids, "view")
@@ -121,7 +132,7 @@ class ViewsAPI(APIClient):
             resource_cls=View,
             identifiers=identifier,
             params={"includeInheritedProperties": include_inherited_properties},
-            executor=get_data_modeling_executor(),
+            executor=ConcurrencySettings.get_data_modeling_executor(),
         )
         if all_versions is True:
             return views
@@ -140,8 +151,8 @@ class ViewsAPI(APIClient):
             Delete views by id::
 
                 >>> from cognite.client import CogniteClient
-                >>> c = CogniteClient()
-                >>> c.data_modeling.views.delete(('mySpace', 'myView', 'v1'))
+                >>> client = CogniteClient()
+                >>> client.data_modeling.views.delete(('mySpace', 'myView', 'v1'))
         """
         deleted_views = cast(
             list,
@@ -149,7 +160,7 @@ class ViewsAPI(APIClient):
                 identifiers=_load_identifier(ids, "view"),
                 wrap_ids=True,
                 returns_items=True,
-                executor=get_data_modeling_executor(),
+                executor=ConcurrencySettings.get_data_modeling_executor(),
             ),
         )
         return [ViewId(item["space"], item["externalId"], item["version"]) for item in deleted_views]
@@ -179,21 +190,21 @@ class ViewsAPI(APIClient):
             List 5 views::
 
                 >>> from cognite.client import CogniteClient
-                >>> c = CogniteClient()
-                >>> view_list = c.data_modeling.views.list(limit=5)
+                >>> client = CogniteClient()
+                >>> view_list = client.data_modeling.views.list(limit=5)
 
             Iterate over views::
 
                 >>> from cognite.client import CogniteClient
-                >>> c = CogniteClient()
-                >>> for view in c.data_modeling.views:
+                >>> client = CogniteClient()
+                >>> for view in client.data_modeling.views:
                 ...     view # do something with the view
 
             Iterate over chunks of views to reduce memory load::
 
                 >>> from cognite.client import CogniteClient
-                >>> c = CogniteClient()
-                >>> for view_list in c.data_modeling.views(chunk_size=10):
+                >>> client = CogniteClient()
+                >>> for view_list in client.data_modeling.views(chunk_size=10):
                 ...     view_list # do something with the views
         """
         filter_ = ViewFilter(space, include_inherited_properties, all_versions, include_global)
@@ -203,12 +214,10 @@ class ViewsAPI(APIClient):
         )
 
     @overload
-    def apply(self, view: Sequence[ViewApply]) -> ViewList:
-        ...
+    def apply(self, view: Sequence[ViewApply]) -> ViewList: ...
 
     @overload
-    def apply(self, view: ViewApply) -> View:
-        ...
+    def apply(self, view: ViewApply) -> View: ...
 
     def apply(self, view: ViewApply | Sequence[ViewApply]) -> View | ViewList:
         """`Create or update (upsert) one or more views. <https://developer.cognite.com/api#tag/Views/operation/ApplyViews>`_
@@ -224,16 +233,80 @@ class ViewsAPI(APIClient):
             Create new views::
 
                 >>> from cognite.client import CogniteClient
-                >>> from cognite.client.data_classes.data_modeling import ViewApply
-                >>> c = CogniteClient()
-                >>> views = [ViewApply(space="mySpace",external_id="myView",version="v1"),
-                ... ViewApply(space="mySpace",external_id="myOtherView",version="v1")]
-                >>> res = c.data_modeling.views.apply(views)
+                >>> from cognite.client.data_classes.data_modeling import ViewApply, MappedPropertyApply, ContainerId
+                >>> client = CogniteClient()
+                >>> views = [
+                ...     ViewApply(
+                ...         space="mySpace",
+                ...         external_id="myView",
+                ...         version="v1",
+                ...         properties={
+                ...             "someAlias": MappedPropertyApply(
+                ...                 container=ContainerId("mySpace", "myContainer"),
+                ...                 container_property_identifier="someProperty",
+                ...             ),
+                ...         }
+                ...    )
+                ... ]
+                >>> res = client.data_modeling.views.apply(views)
+
+
+            Create views with edge relations::
+
+                >>> from cognite.client import CogniteClient
+                >>> from cognite.client.data_classes.data_modeling import (
+                ...     ContainerId,
+                ...     DirectRelationReference,
+                ...     MappedPropertyApply,
+                ...     MultiEdgeConnectionApply,
+                ...     ViewApply,
+                ...     ViewId
+                ... )
+                >>> client = CogniteClient()
+                >>> acts_in_edge_type = DirectRelationReference(space="imdb", external_id="acts-in")
+                >>> movie_view = ViewApply(
+                ...     space="imdb",
+                ...     external_id="Movie",
+                ...     version="1",
+                ...     name="Movie",
+                ...     properties={
+                ...         "title": MappedPropertyApply(
+                ...             container=ContainerId(space="imdb", external_id="Movie"),
+                ...             container_property_identifier="title",
+                ...         ),
+                ...         "actors": MultiEdgeConnectionApply(
+                ...             type=acts_in_edge_type,
+                ...             direction="inwards",
+                ...             source=ViewId("imdb", "Actor", "1"),
+                ...             name="actors",
+                ...         ),
+                ...     }
+                ... )
+                >>> actor_view = ViewApply(
+                ...     space="imdb",
+                ...     external_id="Actor",
+                ...     version="1",
+                ...     name="Actor",
+                ...     properties={
+                ...         "name": MappedPropertyApply(
+                ...             container=ContainerId("imdb", "Actor"),
+                ...             name="name",
+                ...             container_property_identifier="name",
+                ...         ),
+                ...         "movies": MultiEdgeConnectionApply(
+                ...             type=acts_in_edge_type,
+                ...             direction="outwards",
+                ...             source=ViewId("imdb", "Movie", "1"),
+                ...             name="movies",
+                ...         ),
+                ...     }
+                ... )
+                >>> res = client.data_modeling.views.apply([movie_view, actor_view])
         """
         return self._create_multiple(
             list_cls=ViewList,
             resource_cls=View,
             items=view,
             input_resource_cls=ViewApply,
-            executor=get_data_modeling_executor(),
+            executor=ConcurrencySettings.get_data_modeling_executor(),
         )
