@@ -1,16 +1,27 @@
 from __future__ import annotations
 
-import copy
-from typing import TYPE_CHECKING, Any, Iterator, Literal, Sequence, cast, overload
+import itertools
+import warnings
+from collections.abc import Iterator, Sequence
+from functools import partial
+from typing import TYPE_CHECKING, Literal, overload
 
 from cognite.client._api_client import APIClient
 from cognite.client._constants import DEFAULT_LIMIT_READ
-from cognite.client.data_classes import Relationship, RelationshipFilter, RelationshipList, RelationshipUpdate
+from cognite.client.data_classes import (
+    Relationship,
+    RelationshipFilter,
+    RelationshipList,
+    RelationshipUpdate,
+    RelationshipWrite,
+)
 from cognite.client.data_classes.labels import LabelFilter
-from cognite.client.utils._auxiliary import assert_type, is_unlimited
+from cognite.client.data_classes.relationships import RelationshipCore
+from cognite.client.utils._auxiliary import is_unlimited, split_into_chunks
 from cognite.client.utils._concurrency import execute_tasks
 from cognite.client.utils._identifier import IdentifierSequence
-from cognite.client.utils._validation import process_data_set_ids
+from cognite.client.utils._validation import assert_type, process_data_set_ids
+from cognite.client.utils.useful_types import SequenceNotStr
 
 if TYPE_CHECKING:
     from cognite.client import CogniteClient
@@ -24,44 +35,16 @@ class RelationshipsAPI(APIClient):
         super().__init__(config, api_version, cognite_client)
         self._LIST_SUBQUERY_LIMIT = 1000
 
-    def _create_filter(
-        self,
-        source_external_ids: Sequence[str] | None = None,
-        source_types: Sequence[str] | None = None,
-        target_external_ids: Sequence[str] | None = None,
-        target_types: Sequence[str] | None = None,
-        data_set_ids: Sequence[dict[str, Any]] | None = None,
-        start_time: dict[str, int] | None = None,
-        end_time: dict[str, int] | None = None,
-        confidence: dict[str, int] | None = None,
-        last_updated_time: dict[str, int] | None = None,
-        created_time: dict[str, int] | None = None,
-        active_at_time: dict[str, int] | None = None,
-        labels: LabelFilter | None = None,
-    ) -> dict[str, Any]:
-        return RelationshipFilter(
-            source_external_ids=source_external_ids,
-            source_types=source_types,
-            target_external_ids=target_external_ids,
-            target_types=target_types,
-            data_set_ids=data_set_ids,
-            start_time=start_time,
-            end_time=end_time,
-            confidence=confidence,
-            last_updated_time=last_updated_time,
-            created_time=created_time,
-            active_at_time=active_at_time,
-            labels=labels,
-        ).dump(camel_case=True)
-
+    @overload
     def __call__(
         self,
-        source_external_ids: Sequence[str] | None = None,
-        source_types: Sequence[str] | None = None,
-        target_external_ids: Sequence[str] | None = None,
-        target_types: Sequence[str] | None = None,
+        chunk_size: None = None,
+        source_external_ids: SequenceNotStr[str] | None = None,
+        source_types: SequenceNotStr[str] | None = None,
+        target_external_ids: SequenceNotStr[str] | None = None,
+        target_types: SequenceNotStr[str] | None = None,
         data_set_ids: int | Sequence[int] | None = None,
-        data_set_external_ids: str | Sequence[str] | None = None,
+        data_set_external_ids: str | SequenceNotStr[str] | None = None,
         start_time: dict[str, int] | None = None,
         end_time: dict[str, int] | None = None,
         confidence: dict[str, int] | None = None,
@@ -71,7 +54,49 @@ class RelationshipsAPI(APIClient):
         labels: LabelFilter | None = None,
         limit: int | None = None,
         fetch_resources: bool = False,
+        partitions: int | None = None,
+    ) -> Iterator[Relationship]: ...
+
+    @overload
+    def __call__(
+        self,
+        chunk_size: int,
+        source_external_ids: SequenceNotStr[str] | None = None,
+        source_types: SequenceNotStr[str] | None = None,
+        target_external_ids: SequenceNotStr[str] | None = None,
+        target_types: SequenceNotStr[str] | None = None,
+        data_set_ids: int | Sequence[int] | None = None,
+        data_set_external_ids: str | SequenceNotStr[str] | None = None,
+        start_time: dict[str, int] | None = None,
+        end_time: dict[str, int] | None = None,
+        confidence: dict[str, int] | None = None,
+        last_updated_time: dict[str, int] | None = None,
+        created_time: dict[str, int] | None = None,
+        active_at_time: dict[str, int] | None = None,
+        labels: LabelFilter | None = None,
+        limit: int | None = None,
+        fetch_resources: bool = False,
+        partitions: int | None = None,
+    ) -> Iterator[RelationshipList]: ...
+
+    def __call__(
+        self,
         chunk_size: int | None = None,
+        source_external_ids: SequenceNotStr[str] | None = None,
+        source_types: SequenceNotStr[str] | None = None,
+        target_external_ids: SequenceNotStr[str] | None = None,
+        target_types: SequenceNotStr[str] | None = None,
+        data_set_ids: int | Sequence[int] | None = None,
+        data_set_external_ids: str | SequenceNotStr[str] | None = None,
+        start_time: dict[str, int] | None = None,
+        end_time: dict[str, int] | None = None,
+        confidence: dict[str, int] | None = None,
+        last_updated_time: dict[str, int] | None = None,
+        created_time: dict[str, int] | None = None,
+        active_at_time: dict[str, int] | None = None,
+        labels: LabelFilter | None = None,
+        limit: int | None = None,
+        fetch_resources: bool = False,
         partitions: int | None = None,
     ) -> Iterator[Relationship] | Iterator[RelationshipList]:
         """Iterate over relationships
@@ -79,12 +104,13 @@ class RelationshipsAPI(APIClient):
         Fetches relationships as they are iterated over, so you keep a limited number of relationships in memory.
 
         Args:
-            source_external_ids (Sequence[str] | None): Include relationships that have any of these values in their source External Id field
-            source_types (Sequence[str] | None): Include relationships that have any of these values in their source Type field
-            target_external_ids (Sequence[str] | None): Include relationships that have any of these values in their target External Id field
-            target_types (Sequence[str] | None): Include relationships that have any of these values in their target Type field
+            chunk_size (int | None): Number of Relationships to return in each chunk. Defaults to yielding one relationship at a time.
+            source_external_ids (SequenceNotStr[str] | None): Include relationships that have any of these values in their source External Id field
+            source_types (SequenceNotStr[str] | None): Include relationships that have any of these values in their source Type field
+            target_external_ids (SequenceNotStr[str] | None): Include relationships that have any of these values in their target External Id field
+            target_types (SequenceNotStr[str] | None): Include relationships that have any of these values in their target Type field
             data_set_ids (int | Sequence[int] | None): Return only relationships in the specified data set(s) with this id / these ids.
-            data_set_external_ids (str | Sequence[str] | None): Return only relationships in the specified data set(s) with this external id / these external ids.
+            data_set_external_ids (str | SequenceNotStr[str] | None): Return only relationships in the specified data set(s) with this external id / these external ids.
             start_time (dict[str, int] | None): Range between two timestamps, minimum and maximum milliseconds (inclusive)
             end_time (dict[str, int] | None): Range between two timestamps, minimum and maximum milliseconds (inclusive)
             confidence (dict[str, int] | None): Range to filter the field for (inclusive).
@@ -94,15 +120,13 @@ class RelationshipsAPI(APIClient):
             labels (LabelFilter | None): Return only the resource matching the specified label constraints.
             limit (int | None): No description.
             fetch_resources (bool): No description.
-            chunk_size (int | None): Number of Relationships to return in each chunk. Defaults to yielding one relationship at a time.
             partitions (int | None): Retrieve relationships in parallel using this number of workers. Also requires `limit=None` to be passed.
 
         Returns:
             Iterator[Relationship] | Iterator[RelationshipList]: yields Relationship one by one if chunk_size is not specified, else RelationshipList objects.
         """
         data_set_ids_processed = process_data_set_ids(data_set_ids, data_set_external_ids)
-
-        filter = self._create_filter(
+        filter = RelationshipFilter(
             source_external_ids=source_external_ids,
             source_types=source_types,
             target_external_ids=target_external_ids,
@@ -115,16 +139,13 @@ class RelationshipsAPI(APIClient):
             created_time=created_time,
             active_at_time=active_at_time,
             labels=labels,
-        )
-        if (
-            len(filter.get("targetExternalIds", [])) > self._LIST_SUBQUERY_LIMIT
-            or len(filter.get("sourceExternalIds", [])) > self._LIST_SUBQUERY_LIMIT
-        ):
+        ).dump(camel_case=True)
+        n_target_xids, n_source_xids = len(target_external_ids or []), len(source_external_ids or [])
+        if n_target_xids > self._LIST_SUBQUERY_LIMIT or n_source_xids > self._LIST_SUBQUERY_LIMIT:
             raise ValueError(
                 f"For queries with more than {self._LIST_SUBQUERY_LIMIT} source_external_ids or "
-                "target_external_ids, only list is supported"
+                "target_external_ids, only `list` is supported"
             )
-
         return self._list_generator(
             list_cls=RelationshipList,
             resource_cls=Relationship,
@@ -144,7 +165,7 @@ class RelationshipsAPI(APIClient):
         Returns:
             Iterator[Relationship]: yields Relationships one by one.
         """
-        return cast(Iterator[Relationship], self())
+        return self()
 
     def retrieve(self, external_id: str, fetch_resources: bool = False) -> Relationship | None:
         """Retrieve a single relationship by external id.
@@ -161,8 +182,8 @@ class RelationshipsAPI(APIClient):
             Get relationship by external id:
 
                 >>> from cognite.client import CogniteClient
-                >>> c = CogniteClient()
-                >>> res = c.relationships.retrieve(external_id="1")
+                >>> client = CogniteClient()
+                >>> res = client.relationships.retrieve(external_id="1")
         """
         identifiers = IdentifierSequence.load(ids=None, external_ids=external_id).as_singleton()
         return self._retrieve_multiple(
@@ -173,12 +194,12 @@ class RelationshipsAPI(APIClient):
         )
 
     def retrieve_multiple(
-        self, external_ids: Sequence[str], fetch_resources: bool = False, ignore_unknown_ids: bool = False
+        self, external_ids: SequenceNotStr[str], fetch_resources: bool = False, ignore_unknown_ids: bool = False
     ) -> RelationshipList:
         """`Retrieve multiple relationships by external id.  <https://developer.cognite.com/api#tag/Relationships/operation/byidsRelationships>`_
 
         Args:
-            external_ids (Sequence[str]): External IDs
+            external_ids (SequenceNotStr[str]): External IDs
             fetch_resources (bool): if true, will try to return the full resources referenced by the relationship in the
                 source and target fields.
             ignore_unknown_ids (bool): Ignore IDs and external IDs that are not found rather than throw an exception.
@@ -191,8 +212,8 @@ class RelationshipsAPI(APIClient):
             Get relationships by external id::
 
                 >>> from cognite.client import CogniteClient
-                >>> c = CogniteClient()
-                >>> res = c.relationships.retrieve_multiple(external_ids=["abc", "def"])
+                >>> client = CogniteClient()
+                >>> res = client.relationships.retrieve_multiple(external_ids=["abc", "def"])
         """
         identifiers = IdentifierSequence.load(ids=None, external_ids=external_ids)
         return self._retrieve_multiple(
@@ -205,12 +226,12 @@ class RelationshipsAPI(APIClient):
 
     def list(
         self,
-        source_external_ids: Sequence[str] | None = None,
-        source_types: Sequence[str] | None = None,
-        target_external_ids: Sequence[str] | None = None,
-        target_types: Sequence[str] | None = None,
+        source_external_ids: SequenceNotStr[str] | None = None,
+        source_types: SequenceNotStr[str] | None = None,
+        target_external_ids: SequenceNotStr[str] | None = None,
+        target_types: SequenceNotStr[str] | None = None,
         data_set_ids: int | Sequence[int] | None = None,
-        data_set_external_ids: str | Sequence[str] | None = None,
+        data_set_external_ids: str | SequenceNotStr[str] | None = None,
         start_time: dict[str, int] | None = None,
         end_time: dict[str, int] | None = None,
         confidence: dict[str, int] | None = None,
@@ -225,12 +246,12 @@ class RelationshipsAPI(APIClient):
         """`Lists relationships stored in the project based on a query filter given in the payload of this request. Up to 1000 relationships can be retrieved in one operation.  <https://developer.cognite.com/api#tag/Relationships/operation/listRelationships>`_
 
         Args:
-            source_external_ids (Sequence[str] | None): Include relationships that have any of these values in their source External Id field
-            source_types (Sequence[str] | None): Include relationships that have any of these values in their source Type field
-            target_external_ids (Sequence[str] | None): Include relationships that have any of these values in their target External Id field
-            target_types (Sequence[str] | None): Include relationships that have any of these values in their target Type field
+            source_external_ids (SequenceNotStr[str] | None): Include relationships that have any of these values in their source External Id field
+            source_types (SequenceNotStr[str] | None): Include relationships that have any of these values in their source Type field
+            target_external_ids (SequenceNotStr[str] | None): Include relationships that have any of these values in their target External Id field
+            target_types (SequenceNotStr[str] | None): Include relationships that have any of these values in their target Type field
             data_set_ids (int | Sequence[int] | None): Return only relationships in the specified data set(s) with this id / these ids.
-            data_set_external_ids (str | Sequence[str] | None): Return only relationships in the specified data set(s) with this external id / these external ids.
+            data_set_external_ids (str | SequenceNotStr[str] | None): Return only relationships in the specified data set(s) with this external id / these external ids.
             start_time (dict[str, int] | None): Range between two timestamps, minimum and maximum milliseconds (inclusive)
             end_time (dict[str, int] | None): Range between two timestamps, minimum and maximum milliseconds (inclusive)
             confidence (dict[str, int] | None): Range to filter the field for (inclusive).
@@ -250,19 +271,18 @@ class RelationshipsAPI(APIClient):
             List relationships::
 
                 >>> from cognite.client import CogniteClient
-                >>> c = CogniteClient()
-                >>> relationship_list = c.relationships.list(limit=5)
+                >>> client = CogniteClient()
+                >>> relationship_list = client.relationships.list(limit=5)
 
             Iterate over relationships::
 
                 >>> from cognite.client import CogniteClient
-                >>> c = CogniteClient()
-                >>> for relationship in c.relationships:
+                >>> client = CogniteClient()
+                >>> for relationship in client.relationships:
                 ...     relationship # do something with the relationship
         """
         data_set_ids_processed = process_data_set_ids(data_set_ids, data_set_external_ids)
-
-        filter = self._create_filter(
+        filter = RelationshipFilter(
             source_external_ids=source_external_ids,
             source_types=source_types,
             target_external_ids=target_external_ids,
@@ -275,59 +295,73 @@ class RelationshipsAPI(APIClient):
             created_time=created_time,
             active_at_time=active_at_time,
             labels=labels,
-        )
-        target_external_id_list: list[str] = filter.get("targetExternalIds", [])
-        source_external_id_list: list[str] = filter.get("sourceExternalIds", [])
-        if (
-            len(target_external_id_list) > self._LIST_SUBQUERY_LIMIT
-            or len(source_external_id_list) > self._LIST_SUBQUERY_LIMIT
-        ):
-            if not is_unlimited(limit):
-                raise ValueError(
-                    f"Querying more than {self._LIST_SUBQUERY_LIMIT} source_external_ids/target_external_ids only "
-                    f"supported for queries without limit (pass -1 / None / inf instead of {limit})"
-                )
-            tasks = []
+        ).dump(camel_case=True)
 
-            for ti in range(0, max(1, len(target_external_id_list)), self._LIST_SUBQUERY_LIMIT):
-                for si in range(0, max(1, len(source_external_id_list)), self._LIST_SUBQUERY_LIMIT):
-                    task_filter = copy.copy(filter)
-                    if target_external_id_list:  # keep null if it was
-                        task_filter["targetExternalIds"] = target_external_id_list[ti : ti + self._LIST_SUBQUERY_LIMIT]
-                    if source_external_id_list:  # keep null if it was
-                        task_filter["sourceExternalIds"] = source_external_id_list[si : si + self._LIST_SUBQUERY_LIMIT]
-                    tasks.append((task_filter,))
-
-            tasks_summary = execute_tasks(
-                lambda filter: self._list(
-                    list_cls=RelationshipList,
-                    resource_cls=Relationship,
-                    method="POST",
-                    limit=limit,
-                    filter=filter,
-                    other_params={"fetchResources": fetch_resources},
-                    partitions=partitions,
-                ),
-                tasks,
-                max_workers=self._config.max_workers,
+        target_external_ids, source_external_ids = target_external_ids or [], source_external_ids or []
+        if all(len(xids) <= self._LIST_SUBQUERY_LIMIT for xids in (target_external_ids, source_external_ids)):
+            return self._list(
+                list_cls=RelationshipList,
+                resource_cls=Relationship,
+                method="POST",
+                limit=limit,
+                filter=filter,
+                partitions=partitions,
+                other_params={"fetchResources": fetch_resources},
             )
-            tasks_summary.raise_first_encountered_exception()
+        if not is_unlimited(limit):
+            raise ValueError(
+                f"Querying more than {self._LIST_SUBQUERY_LIMIT} source_external_ids/target_external_ids is only "
+                f"supported for unlimited queries (pass -1 / None / inf instead of {limit})"
+            )
+        tasks = []
+        target_chunks = split_into_chunks(target_external_ids, self._LIST_SUBQUERY_LIMIT) or [[]]
+        source_chunks = split_into_chunks(source_external_ids, self._LIST_SUBQUERY_LIMIT) or [[]]
 
-            return RelationshipList(tasks_summary.joined_results())
-        return self._list(
-            list_cls=RelationshipList,
-            resource_cls=Relationship,
-            method="POST",
-            limit=limit,
-            filter=filter,
-            other_params={"fetchResources": fetch_resources},
+        # All sources (if any) must be checked against all targets (if any). When either is not
+        # given, we must exhaustively list all matching just the source or the target:
+        for target_xids, source_xids in itertools.product(target_chunks, source_chunks):
+            task_filter = filter.copy()
+            if target_external_ids:  # keep null if it was
+                task_filter["targetExternalIds"] = target_xids
+            if source_external_ids:
+                task_filter["sourceExternalIds"] = source_xids
+            tasks.append({"filter": task_filter})
+
+        if partitions is not None:
+            warnings.warn(
+                f"When one or both of source/target external IDs have more than {self._LIST_SUBQUERY_LIMIT} "
+                "elements, `partitions` is ignored",
+                UserWarning,
+            )
+        tasks_summary = execute_tasks(
+            partial(
+                self._list,
+                list_cls=RelationshipList,
+                resource_cls=Relationship,
+                method="POST",
+                limit=None,
+                partitions=None,  # Otherwise, workers will spawn workers -> deadlock (singleton threadpool)
+                other_params={"fetchResources": fetch_resources},
+            ),
+            tasks,
+            max_workers=self._config.max_workers,
         )
+        tasks_summary.raise_compound_exception_if_failed_tasks()
+        return RelationshipList(tasks_summary.joined_results(), cognite_client=self._cognite_client)
 
-    def create(self, relationship: Relationship | Sequence[Relationship]) -> Relationship | RelationshipList:
+    @overload
+    def create(self, relationship: Relationship | RelationshipWrite) -> Relationship: ...
+
+    @overload
+    def create(self, relationship: Sequence[Relationship | RelationshipWrite]) -> RelationshipList: ...
+
+    def create(
+        self, relationship: Relationship | RelationshipWrite | Sequence[Relationship | RelationshipWrite]
+    ) -> Relationship | RelationshipList:
         """`Create one or more relationships. <https://developer.cognite.com/api#tag/Relationships/operation/createRelationships>`_
 
         Args:
-            relationship (Relationship | Sequence[Relationship]): Relationship or list of relationships to create.
+            relationship (Relationship | RelationshipWrite | Sequence[Relationship | RelationshipWrite]): Relationship or list of relationships to create.
 
         Returns:
             Relationship | RelationshipList: Created relationship(s)
@@ -342,7 +376,7 @@ class RelationshipsAPI(APIClient):
 
                 >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import Relationship
-                >>> c = CogniteClient()
+                >>> client = CogniteClient()
                 >>> flowrel1 = Relationship(
                 ...     external_id="flow_1",
                 ...     source_external_id="source_ext_id",
@@ -361,24 +395,41 @@ class RelationshipsAPI(APIClient):
                 ...     confidence=0.1,
                 ...     data_set_id=1234
                 ... )
-                >>> res = c.relationships.create([flowrel1,flowrel2])
+                >>> res = client.relationships.create([flowrel1,flowrel2])
         """
-        assert_type(relationship, "relationship", [Relationship, Sequence])
+        assert_type(relationship, "relationship", [RelationshipCore, Sequence])
         if isinstance(relationship, Sequence):
             relationship = [r._validate_resource_types() for r in relationship]
         else:
             relationship = relationship._validate_resource_types()
 
-        return self._create_multiple(list_cls=RelationshipList, resource_cls=Relationship, items=relationship)
+        return self._create_multiple(
+            list_cls=RelationshipList,
+            resource_cls=Relationship,
+            items=relationship,
+            input_resource_cls=RelationshipWrite,
+        )
+
+    @overload
+    def update(self, item: Relationship | RelationshipWrite | RelationshipUpdate) -> Relationship: ...
+
+    @overload
+    def update(self, item: Sequence[Relationship | RelationshipWrite | RelationshipUpdate]) -> RelationshipList: ...
 
     def update(
-        self, item: Relationship | RelationshipUpdate | Sequence[Relationship | RelationshipUpdate]
+        self,
+        item: Relationship
+        | RelationshipWrite
+        | RelationshipUpdate
+        | Sequence[Relationship | RelationshipWrite | RelationshipUpdate],
+        mode: Literal["replace_ignore_null", "patch", "replace"] = "replace_ignore_null",
     ) -> Relationship | RelationshipList:
         """`Update one or more relationships <https://developer.cognite.com/api#tag/Relationships/operation/updateRelationships>`_
         Currently, a full replacement of labels on a relationship is not supported (only partial add/remove updates). See the example below on how to perform partial labels update.
 
         Args:
-            item (Relationship | RelationshipUpdate | Sequence[Relationship | RelationshipUpdate]): Relationship(s) to update
+            item (Relationship | RelationshipWrite | RelationshipUpdate | Sequence[Relationship | RelationshipWrite | RelationshipUpdate]): Relationship(s) to update
+            mode (Literal['replace_ignore_null', 'patch', 'replace']): How to update data when a non-update object is given (Relationship or -Write). If you use 'replace_ignore_null', only the fields you have set will be used to replace existing (default). Using 'replace' will additionally clear all the fields that are not specified by you. Last option, 'patch', will update only the fields you have set and for container-like fields such as metadata or labels, add the values to the existing. For more details, see :ref:`appendix-update`.
 
         Returns:
             Relationship | RelationshipList: Updated relationship(s)
@@ -387,52 +438,56 @@ class RelationshipsAPI(APIClient):
             Update a data set that you have fetched. This will perform a full update of the data set::
 
                 >>> from cognite.client import CogniteClient
-                >>> c = CogniteClient()
-                >>> rel = c.relationships.retrieve(external_id="flow1")
+                >>> client = CogniteClient()
+                >>> rel = client.relationships.retrieve(external_id="flow1")
                 >>> rel.confidence = 0.75
-                >>> res = c.relationships.update(rel)
+                >>> res = client.relationships.update(rel)
 
             Perform a partial update on a relationship, setting a source_external_id and a confidence::
 
                 >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import RelationshipUpdate
-                >>> c = CogniteClient()
+                >>> client = CogniteClient()
                 >>> my_update = RelationshipUpdate(external_id="flow_1").source_external_id.set("alternate_source").confidence.set(0.97)
-                >>> res1 = c.relationships.update(my_update)
+                >>> res1 = client.relationships.update(my_update)
                 >>> # Remove an already set optional field like so
                 >>> another_update = RelationshipUpdate(external_id="flow_1").confidence.set(None)
-                >>> res2 = c.relationships.update(another_update)
+                >>> res2 = client.relationships.update(another_update)
 
             Attach labels to a relationship::
 
                 >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import RelationshipUpdate
-                >>> c = CogniteClient()
+                >>> client = CogniteClient()
                 >>> my_update = RelationshipUpdate(external_id="flow_1").labels.add(["PUMP", "VERIFIED"])
-                >>> res = c.relationships.update(my_update)
+                >>> res = client.relationships.update(my_update)
 
             Detach a single label from a relationship::
 
                 >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import RelationshipUpdate
-                >>> c = CogniteClient()
+                >>> client = CogniteClient()
                 >>> my_update = RelationshipUpdate(external_id="flow_1").labels.remove("PUMP")
-                >>> res = c.relationships.update(my_update)
+                >>> res = client.relationships.update(my_update)
         """
         return self._update_multiple(
-            list_cls=RelationshipList, resource_cls=Relationship, update_cls=RelationshipUpdate, items=item
+            list_cls=RelationshipList, resource_cls=Relationship, update_cls=RelationshipUpdate, items=item, mode=mode
         )
 
     @overload
-    def upsert(self, item: Sequence[Relationship], mode: Literal["patch", "replace"] = "patch") -> RelationshipList:
-        ...
+    def upsert(
+        self, item: Sequence[Relationship | RelationshipWrite], mode: Literal["patch", "replace"] = "patch"
+    ) -> RelationshipList: ...
 
     @overload
-    def upsert(self, item: Relationship, mode: Literal["patch", "replace"] = "patch") -> Relationship:
-        ...
+    def upsert(
+        self, item: Relationship | RelationshipWrite, mode: Literal["patch", "replace"] = "patch"
+    ) -> Relationship: ...
 
     def upsert(
-        self, item: Relationship | Sequence[Relationship], mode: Literal["patch", "replace"] = "patch"
+        self,
+        item: Relationship | RelationshipWrite | Sequence[Relationship | RelationshipWrite],
+        mode: Literal["patch", "replace"] = "patch",
     ) -> Relationship | RelationshipList:
         """Upsert relationships, i.e., update if it exists, and create if it does not exist.
             Note this is a convenience method that handles the upserting for you by first calling update on all items,
@@ -441,8 +496,8 @@ class RelationshipsAPI(APIClient):
             For more details, see :ref:`appendix-upsert`.
 
         Args:
-            item (Relationship | Sequence[Relationship]): Relationship or list of relationships to upsert.
-            mode (Literal["patch", "replace"]): Whether to patch or replace in the case the relationships are existing. If you set 'patch', the call will only update fields with non-null values (default). Setting 'replace' will unset any fields that are not specified.
+            item (Relationship | RelationshipWrite | Sequence[Relationship | RelationshipWrite]): Relationship or list of relationships to upsert.
+            mode (Literal['patch', 'replace']): Whether to patch or replace in the case the relationships are existing. If you set 'patch', the call will only update fields with non-null values (default). Setting 'replace' will unset any fields that are not specified.
 
         Returns:
             Relationship | RelationshipList: The upserted relationship(s).
@@ -453,11 +508,11 @@ class RelationshipsAPI(APIClient):
 
                 >>> from cognite.client import CogniteClient
                 >>> from cognite.client.data_classes import Relationship
-                >>> c = CogniteClient()
-                >>> existing_relationship = c.relationships.retrieve(id=1)
+                >>> client = CogniteClient()
+                >>> existing_relationship = client.relationships.retrieve(id=1)
                 >>> existing_relationship.description = "New description"
                 >>> new_relationship = Relationship(external_id="new_relationship", source_external_id="new_source")
-                >>> res = c.relationships.upsert([existing_relationship, new_relationship], mode="replace")
+                >>> res = client.relationships.upsert([existing_relationship, new_relationship], mode="replace")
         """
         return self._upsert_multiple(
             item,
@@ -468,19 +523,19 @@ class RelationshipsAPI(APIClient):
             mode=mode,
         )
 
-    def delete(self, external_id: str | Sequence[str], ignore_unknown_ids: bool = False) -> None:
+    def delete(self, external_id: str | SequenceNotStr[str], ignore_unknown_ids: bool = False) -> None:
         """`Delete one or more relationships. <https://developer.cognite.com/api#tag/Relationships/operation/deleteRelationships>`_
 
         Args:
-            external_id (str | Sequence[str]): External ID or list of external ids
+            external_id (str | SequenceNotStr[str]): External ID or list of external ids
             ignore_unknown_ids (bool): Ignore external IDs that are not found rather than throw an exception.
         Examples:
 
             Delete relationships by external id::
 
                 >>> from cognite.client import CogniteClient
-                >>> c = CogniteClient()
-                >>> c.relationships.delete(external_id=["a","b"])
+                >>> client = CogniteClient()
+                >>> client.relationships.delete(external_id=["a","b"])
         """
         self._delete_multiple(
             identifiers=IdentifierSequence.load(external_ids=external_id),
