@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from typing import Sequence
+from collections.abc import Sequence
 
 from cognite.client._api_client import APIClient
 from cognite.client._constants import DEFAULT_LIMIT_READ
@@ -39,14 +39,14 @@ class TransformationJobsAPI(APIClient):
             List transformation jobs::
 
                 >>> from cognite.client import CogniteClient
-                >>> c = CogniteClient()
-                >>> transformation_jobs_list = c.transformations.jobs.list()
+                >>> client = CogniteClient()
+                >>> transformation_jobs_list = client.transformations.jobs.list()
 
             List transformation jobs of a single transformation::
 
                 >>> from cognite.client import CogniteClient
-                >>> c = CogniteClient()
-                >>> transformation_jobs_list = c.transformations.jobs.list(transformation_id = 1)
+                >>> client = CogniteClient()
+                >>> transformation_jobs_list = client.transformations.jobs.list(transformation_id=1)
         """
 
         filter = TransformationJobFilter(
@@ -71,8 +71,8 @@ class TransformationJobsAPI(APIClient):
             Get transformation job by id:
 
                 >>> from cognite.client import CogniteClient
-                >>> c = CogniteClient()
-                >>> res = c.transformations.jobs.retrieve(id=1)
+                >>> client = CogniteClient()
+                >>> res = client.transformations.jobs.retrieve(id=1)
         """
         identifiers = IdentifierSequence.load(ids=id, external_ids=None).as_singleton()
         return self._retrieve_multiple(
@@ -93,8 +93,8 @@ class TransformationJobsAPI(APIClient):
             Get metrics by transformation job id:
 
                 >>> from cognite.client import CogniteClient
-                >>> c = CogniteClient()
-                >>> res = c.transformations.jobs.list_metrics(id=1)
+                >>> client = CogniteClient()
+                >>> res = client.transformations.jobs.list_metrics(id=1)
         """
         url_path = interpolate_and_url_encode(self._RESOURCE_PATH + "/{}/metrics", str(id))
 
@@ -121,8 +121,8 @@ class TransformationJobsAPI(APIClient):
             Get jobs by id::
 
                 >>> from cognite.client import CogniteClient
-                >>> c = CogniteClient()
-                >>> res = c.transformations.jobs.retrieve_multiple(ids=[1, 2, 3])
+                >>> client = CogniteClient()
+                >>> res = client.transformations.jobs.retrieve_multiple(ids=[1, 2, 3])
         """
         identifiers = IdentifierSequence.load(ids=ids, external_ids=None)
         return self._retrieve_multiple(
