@@ -1,20 +1,25 @@
 from __future__ import annotations
 
-from cognite.client.data_classes._base import Geometry
+from cognite.client.data_classes._base import Geometry, HasExternalAndInternalId, HasExternalId, HasInternalId, HasName
+from cognite.client.data_classes.aggregations import CountAggregate
 from cognite.client.data_classes.annotations import (
     Annotation,
     AnnotationFilter,
     AnnotationList,
+    AnnotationReverseLookupFilter,
     AnnotationUpdate,
+    AnnotationWrite,
+    AnnotationWriteList,
 )
 from cognite.client.data_classes.assets import (
     AggregateResultItem,
     Asset,
-    AssetAggregate,
     AssetFilter,
     AssetHierarchy,
     AssetList,
     AssetUpdate,
+    AssetWrite,
+    AssetWriteList,
 )
 from cognite.client.data_classes.contextualization import (
     ContextualizationJob,
@@ -27,10 +32,11 @@ from cognite.client.data_classes.contextualization import (
 )
 from cognite.client.data_classes.data_sets import (
     DataSet,
-    DataSetAggregate,
     DataSetFilter,
     DataSetList,
     DataSetUpdate,
+    DataSetWrite,
+    DataSetWriteList,
 )
 from cognite.client.data_classes.datapoints import (
     Datapoint,
@@ -38,13 +44,17 @@ from cognite.client.data_classes.datapoints import (
     DatapointsArray,
     DatapointsArrayList,
     DatapointsList,
+    DatapointsQuery,
     LatestDatapointQuery,
+    StatusCode,
 )
 from cognite.client.data_classes.datapoints_subscriptions import (
     DatapointSubscription,
     DataPointSubscriptionCreate,
     DatapointSubscriptionList,
     DataPointSubscriptionUpdate,
+    DataPointSubscriptionWrite,
+    DatapointSubscriptionWriteList,
 )
 from cognite.client.data_classes.documents import (
     Document,
@@ -53,25 +63,41 @@ from cognite.client.data_classes.documents import (
     DocumentList,
     SourceFile,
 )
-from cognite.client.data_classes.events import EndTimeFilter, Event, EventFilter, EventList, EventUpdate
+from cognite.client.data_classes.events import (
+    EndTimeFilter,
+    Event,
+    EventFilter,
+    EventList,
+    EventUpdate,
+    EventWrite,
+    EventWriteList,
+)
 from cognite.client.data_classes.extractionpipelines import (
     ExtractionPipeline,
     ExtractionPipelineConfig,
     ExtractionPipelineConfigRevision,
     ExtractionPipelineConfigRevisionList,
+    ExtractionPipelineConfigWrite,
+    ExtractionPipelineConfigWriteList,
     ExtractionPipelineContact,
     ExtractionPipelineList,
     ExtractionPipelineRun,
     ExtractionPipelineRunFilter,
     ExtractionPipelineRunList,
+    ExtractionPipelineRunWrite,
+    ExtractionPipelineRunWriteList,
     ExtractionPipelineUpdate,
+    ExtractionPipelineWrite,
+    ExtractionPipelineWriteList,
 )
 from cognite.client.data_classes.files import (
-    FileAggregate,
     FileMetadata,
     FileMetadataFilter,
     FileMetadataList,
     FileMetadataUpdate,
+    FileMetadataWrite,
+    FileMetadataWriteList,
+    FileMultipartUploadSession,
 )
 from cognite.client.data_classes.functions import (
     Function,
@@ -84,11 +110,17 @@ from cognite.client.data_classes.functions import (
     FunctionSchedule,
     FunctionSchedulesFilter,
     FunctionSchedulesList,
+    FunctionScheduleWrite,
+    FunctionScheduleWriteList,
     FunctionsLimits,
+    FunctionWrite,
+    FunctionWriteList,
 )
 from cognite.client.data_classes.geospatial import (
     CoordinateReferenceSystem,
     CoordinateReferenceSystemList,
+    CoordinateReferenceSystemWrite,
+    CoordinateReferenceSystemWriteList,
     Feature,
     FeatureAggregate,
     FeatureAggregateList,
@@ -96,16 +128,23 @@ from cognite.client.data_classes.geospatial import (
     FeatureType,
     FeatureTypeList,
     FeatureTypePatch,
-    FeatureTypeUpdate,
-    FeatureTypeUpdateList,
+    FeatureTypeWrite,
+    FeatureTypeWriteList,
+    FeatureWrite,
+    FeatureWriteList,
 )
 from cognite.client.data_classes.iam import (
+    ALL_USER_ACCOUNTS,
     ClientCredentials,
     CreatedSession,
     Group,
     GroupList,
+    GroupWrite,
+    GroupWriteList,
     SecurityCategory,
     SecurityCategoryList,
+    SecurityCategoryWrite,
+    SecurityCategoryWriteList,
     Session,
     SessionList,
 )
@@ -114,24 +153,47 @@ from cognite.client.data_classes.labels import (
     LabelDefinition,
     LabelDefinitionFilter,
     LabelDefinitionList,
+    LabelDefinitionWrite,
     LabelFilter,
 )
-from cognite.client.data_classes.raw import Database, DatabaseList, Row, RowList, Table, TableList
+from cognite.client.data_classes.raw import (
+    Database,
+    DatabaseList,
+    DatabaseWrite,
+    DatabaseWriteList,
+    Row,
+    RowList,
+    RowWrite,
+    RowWriteList,
+    Table,
+    TableList,
+    TableWrite,
+    TableWriteList,
+)
 from cognite.client.data_classes.relationships import (
     Relationship,
     RelationshipFilter,
     RelationshipList,
     RelationshipUpdate,
+    RelationshipWrite,
+    RelationshipWriteList,
 )
 from cognite.client.data_classes.sequences import (
     Sequence,
-    SequenceAggregate,
+    SequenceColumn,
+    SequenceColumnList,
     SequenceColumnUpdate,
+    SequenceColumnWrite,
+    SequenceColumnWriteList,
     SequenceData,
-    SequenceDataList,
     SequenceFilter,
     SequenceList,
+    SequenceRow,
+    SequenceRows,
+    SequenceRowsList,
     SequenceUpdate,
+    SequenceWrite,
+    SequenceWriteList,
 )
 from cognite.client.data_classes.shared import (
     AggregateResult,
@@ -159,27 +221,36 @@ from cognite.client.data_classes.three_d import (
     RevisionCameraProperties,
     ThreeDAssetMapping,
     ThreeDAssetMappingList,
+    ThreeDAssetMappingWrite,
+    ThreeDAssetMappingWriteList,
     ThreeDModel,
     ThreeDModelList,
     ThreeDModelRevision,
     ThreeDModelRevisionList,
     ThreeDModelRevisionUpdate,
+    ThreeDModelRevisionWrite,
+    ThreeDModelRevisionWriteList,
     ThreeDModelUpdate,
+    ThreeDModelWrite,
+    ThreeDModelWriteList,
     ThreeDNode,
     ThreeDNodeList,
 )
 from cognite.client.data_classes.time_series import (
     TimeSeries,
-    TimeSeriesAggregate,
     TimeSeriesFilter,
     TimeSeriesList,
     TimeSeriesUpdate,
+    TimeSeriesWrite,
+    TimeSeriesWriteList,
 )
 from cognite.client.data_classes.transformations import (
     Transformation,
     TransformationList,
     TransformationPreviewResult,
     TransformationUpdate,
+    TransformationWrite,
+    TransformationWriteList,
 )
 from cognite.client.data_classes.transformations.common import (
     OidcCredentials,
@@ -198,11 +269,15 @@ from cognite.client.data_classes.transformations.jobs import (
 from cognite.client.data_classes.transformations.notifications import (
     TransformationNotification,
     TransformationNotificationList,
+    TransformationNotificationWrite,
+    TransformationNotificationWriteList,
 )
 from cognite.client.data_classes.transformations.schedules import (
     TransformationSchedule,
     TransformationScheduleList,
     TransformationScheduleUpdate,
+    TransformationScheduleWrite,
+    TransformationScheduleWriteList,
 )
 from cognite.client.data_classes.transformations.schema import (
     TransformationSchemaColumn,
@@ -216,6 +291,7 @@ from cognite.client.data_classes.workflows import (
     DynamicTaskParameters,
     FunctionTaskOutput,
     FunctionTaskParameters,
+    SubworkflowTaskParameters,
     TransformationTaskOutput,
     TransformationTaskParameters,
     Workflow,
@@ -227,24 +303,38 @@ from cognite.client.data_classes.workflows import (
     WorkflowList,
     WorkflowTask,
     WorkflowTaskExecution,
+    WorkflowTrigger,
+    WorkflowTriggerCreate,
+    WorkflowTriggerList,
+    WorkflowTriggerRun,
+    WorkflowTriggerRunList,
+    WorkflowTriggerUpsert,
+    WorkflowTriggerUpsertList,
     WorkflowUpsert,
+    WorkflowUpsertList,
     WorkflowVersion,
     WorkflowVersionId,
     WorkflowVersionList,
     WorkflowVersionUpsert,
+    WorkflowVersionUpsertList,
 )
 
 __all__ = [
     "Annotation",
+    "AnnotationWrite",
+    "AnnotationWriteList",
     "AnnotationFilter",
+    "AnnotationReverseLookupFilter",
     "AnnotationList",
     "AnnotationUpdate",
     "AggregateResultItem",
     "Asset",
-    "AssetAggregate",
+    "AssetWrite",
+    "CountAggregate",
     "AssetFilter",
     "AssetHierarchy",
     "AssetList",
+    "AssetWriteList",
     "AssetUpdate",
     "ContextualizationJob",
     "ContextualizationJobList",
@@ -254,61 +344,92 @@ __all__ = [
     "EntityMatchingModelUpdate",
     "JobStatus",
     "Database",
+    "DatabaseWrite",
     "DatabaseList",
+    "DatabaseWriteList",
     "Row",
+    "RowWrite",
     "RowList",
+    "RowWriteList",
     "Table",
+    "TableWrite",
     "TableList",
+    "TableWriteList",
     "ExtractionPipeline",
+    "ExtractionPipelineWrite",
+    "ExtractionPipelineWriteList",
     "ExtractionPipelineConfig",
+    "ExtractionPipelineConfigWrite",
     "ExtractionPipelineConfigRevision",
     "ExtractionPipelineConfigRevisionList",
+    "ExtractionPipelineConfigWriteList",
     "ExtractionPipelineContact",
     "ExtractionPipelineList",
     "ExtractionPipelineRun",
+    "ExtractionPipelineRunWrite",
     "ExtractionPipelineRunFilter",
     "ExtractionPipelineRunList",
+    "ExtractionPipelineRunWriteList",
     "ExtractionPipelineUpdate",
     "EndTimeFilter",
     "Event",
+    "EventWrite",
     "EventFilter",
     "EventList",
+    "EventWriteList",
     "EventUpdate",
-    "FileAggregate",
     "FileMetadata",
+    "FileMetadataWrite",
     "FileMetadataFilter",
     "FileMetadataList",
+    "FileMetadataWriteList",
     "FileMetadataUpdate",
+    "FileMultipartUploadSession",
     "SourceFile",
     "Document",
     "DocumentList",
     "DocumentHighlight",
     "DocumentHighlightList",
     "ClientCredentials",
+    "ALL_USER_ACCOUNTS",
     "CreatedSession",
     "Group",
+    "GroupWrite",
     "GroupList",
+    "GroupWriteList",
     "SecurityCategory",
+    "SecurityCategoryWrite",
     "SecurityCategoryList",
+    "SecurityCategoryWriteList",
     "Session",
     "SessionList",
     "Label",
     "LabelDefinition",
+    "LabelDefinitionWrite",
     "LabelDefinitionFilter",
     "LabelDefinitionList",
     "LabelFilter",
     "Relationship",
+    "RelationshipWrite",
     "RelationshipFilter",
     "RelationshipList",
+    "RelationshipWriteList",
     "RelationshipUpdate",
     "Sequence",
-    "SequenceAggregate",
+    "SequenceWrite",
     "SequenceColumnUpdate",
     "SequenceData",
-    "SequenceDataList",
+    "SequenceRows",
+    "SequenceRowsList",
     "SequenceFilter",
     "SequenceList",
+    "SequenceWriteList",
     "SequenceUpdate",
+    "SequenceColumn",
+    "SequenceColumnWrite",
+    "SequenceColumnList",
+    "SequenceColumnWriteList",
+    "SequenceRow",
     "ConstantResolver",
     "Source",
     "TemplateGroup",
@@ -323,30 +444,41 @@ __all__ = [
     "BoundingBox3D",
     "RevisionCameraProperties",
     "ThreeDAssetMapping",
+    "ThreeDAssetMappingWrite",
     "ThreeDAssetMappingList",
+    "ThreeDAssetMappingWriteList",
     "ThreeDModel",
+    "ThreeDModelWrite",
     "ThreeDModelList",
+    "ThreeDModelWriteList",
     "ThreeDModelRevision",
+    "ThreeDModelRevisionWrite",
     "ThreeDModelRevisionList",
+    "ThreeDModelRevisionWriteList",
     "ThreeDModelRevisionUpdate",
     "ThreeDModelUpdate",
     "ThreeDNode",
     "ThreeDNodeList",
     "TimeSeries",
-    "TimeSeriesAggregate",
+    "TimeSeriesWrite",
     "TimeSeriesFilter",
     "TimeSeriesList",
+    "TimeSeriesWriteList",
     "TimeSeriesUpdate",
     "DatapointSubscription",
+    "DataPointSubscriptionWrite",
     "DataPointSubscriptionCreate",
     "DatapointSubscriptionList",
     "DataPointSubscriptionUpdate",
+    "DatapointSubscriptionWriteList",
     "OidcCredentials",
     "RawTable",
     "Transformation",
+    "TransformationWrite",
     "TransformationBlockedInfo",
     "TransformationDestination",
     "TransformationList",
+    "TransformationWriteList",
     "TransformationPreviewResult",
     "TransformationUpdate",
     "TransformationJob",
@@ -357,15 +489,20 @@ __all__ = [
     "TransformationJobStatus",
     "TransformationNotification",
     "TransformationNotificationList",
+    "TransformationNotificationWrite",
+    "TransformationNotificationWriteList",
     "TransformationSchedule",
+    "TransformationScheduleWrite",
     "TransformationScheduleList",
+    "TransformationScheduleWriteList",
     "TransformationScheduleUpdate",
     "TransformationSchemaColumn",
     "TransformationSchemaColumnList",
     "DataSet",
-    "DataSetAggregate",
+    "DataSetWrite",
     "DataSetFilter",
     "DataSetList",
+    "DataSetWriteList",
     "DataSetUpdate",
     "AggregateResult",
     "AggregateUniqueValuesResult",
@@ -379,28 +516,38 @@ __all__ = [
     "DatapointsList",
     "DatapointsArray",
     "DatapointsArrayList",
+    "DatapointsQuery",
     "LatestDatapointQuery",
+    "StatusCode",
     "Function",
+    "FunctionWrite",
     "FunctionFilter",
     "FunctionSchedule",
+    "FunctionScheduleWrite",
     "FunctionSchedulesFilter",
+    "FunctionScheduleWriteList",
     "FunctionSchedulesList",
     "FunctionList",
+    "FunctionWriteList",
     "FunctionCall",
     "FunctionCallList",
     "FunctionCallLogEntry",
     "FunctionCallLog",
     "FunctionsLimits",
     "Feature",
+    "FeatureWrite",
     "FeatureList",
+    "FeatureWriteList",
     "FeatureType",
+    "FeatureTypeWrite",
     "FeatureTypeList",
+    "FeatureTypeWriteList",
     "FeatureTypePatch",
     "FeatureAggregate",
-    "FeatureTypeUpdate",
     "FeatureAggregateList",
-    "FeatureTypeUpdateList",
     "CoordinateReferenceSystemList",
+    "CoordinateReferenceSystemWrite",
+    "CoordinateReferenceSystemWriteList",
     "CoordinateReferenceSystem",
     "UserProfile",
     "UserProfileList",
@@ -417,6 +564,7 @@ __all__ = [
     "TransformationTaskParameters",
     "CDFTaskParameters",
     "DynamicTaskParameters",
+    "SubworkflowTaskParameters",
     "FunctionTaskOutput",
     "TransformationTaskOutput",
     "CDFTaskOutput",
@@ -426,4 +574,18 @@ __all__ = [
     "WorkflowTaskExecution",
     "Workflow",
     "WorkflowTask",
+    "WorkflowUpsertList",
+    "WorkflowVersionUpsertList",
+    "WorkflowVersionUpsertList",
+    "WorkflowTrigger",
+    "WorkflowTriggerCreate",
+    "WorkflowTriggerUpsert",
+    "WorkflowTriggerList",
+    "WorkflowTriggerUpsertList",
+    "WorkflowTriggerRun",
+    "WorkflowTriggerRunList",
+    "HasName",
+    "HasExternalId",
+    "HasInternalId",
+    "HasExternalAndInternalId",
 ]
