@@ -2,26 +2,35 @@ from __future__ import annotations
 
 import time
 import warnings
+from collections.abc import Sequence
 from dataclasses import dataclass
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Sequence, Type, TypeVar, Union, cast
+from typing import TYPE_CHECKING, Any, TypeVar, cast
 
 from requests.utils import CaseInsensitiveDict
+from typing_extensions import Self
 
 from cognite.client.data_classes import Annotation
 from cognite.client.data_classes._base import (
+    CogniteObject,
     CognitePrimitiveUpdate,
     CogniteResource,
     CogniteResourceList,
     CogniteUpdate,
     PropertySpec,
 )
-from cognite.client.data_classes.annotation_types.images import AssetLink, ObjectDetection, TextRegion
+from cognite.client.data_classes.annotation_types.images import (
+    AssetLink,
+    KeypointCollectionWithObjectDetection,
+    ObjectDetection,
+    TextRegion,
+)
 from cognite.client.data_classes.annotation_types.primitives import VisionResource
 from cognite.client.data_classes.annotations import AnnotationList
+from cognite.client.data_classes.data_modeling import NodeId
 from cognite.client.exceptions import CogniteAPIError, CogniteException, ModelFailedException
-from cognite.client.utils._auxiliary import convert_true_match, exactly_one_is_not_none
-from cognite.client.utils._text import to_snake_case
+from cognite.client.utils._auxiliary import convert_true_match, exactly_one_is_not_none, load_resource
+from cognite.client.utils._text import convert_all_keys_to_snake_case, to_camel_case, to_snake_case
 
 if TYPE_CHECKING:
     import pandas
@@ -196,9 +205,14 @@ class EntityMatchingModel(CogniteResource):
         external_id: str | None = None,
         cognite_client: CogniteClient | None = None,
     ) -> None:
-        self.id = id
+        # id/created_time are required when using the class to read,
+        # but don't make sense passing in when creating a new object. So in order to make the typing
+        # correct here (i.e. int and not Optional[int]), we force the type to be int rather than
+        # Optional[int].
+        # TODO: In the next major version we can make these properties required in the constructor
+        self.id: int = id  # type: ignore
+        self.created_time: int = created_time  # type: ignore
         self.status = status
-        self.created_time = created_time
         self.start_time = start_time
         self.status_time = status_time
         self.error_message = error_message
@@ -257,7 +271,7 @@ class EntityMatchingModel(CogniteResource):
 
         Args:
             sources (list[dict] | None): entities to match from, does not need an 'id' field. Tolerant to passing more than is needed or used (e.g. json dump of time series list). If omitted, will use data from fit.
-            targets (list[dict] | None): entities to match to, does not need an 'id' field.  Tolerant to passing more than is needed or used. If omitted, will use data from fit.
+            targets (list[dict] | None): entities to match to, does not need an 'id' field. Tolerant to passing more than is needed or used. If omitted, will use data from fit.
             num_matches (int): number of matches to return for each item.
             score_threshold (float | None): only return matches with a score above this threshold
 
@@ -328,7 +342,7 @@ class EntityMatchingModelUpdate(CogniteUpdate):
         return EntityMatchingModelUpdate._PrimitiveUpdate(self, "description")
 
     @classmethod
-    def _get_update_properties(cls) -> list[PropertySpec]:
+    def _get_update_properties(cls, item: CogniteResource | None = None) -> list[PropertySpec]:
         return [
             PropertySpec("name", is_nullable=False),
             PropertySpec("description", is_nullable=False),
@@ -344,24 +358,29 @@ class FileReference:
         self,
         file_id: int | None = None,
         file_external_id: str | None = None,
+        file_instance_id: NodeId | None = None,
         first_page: int | None = None,
         last_page: int | None = None,
     ) -> None:
         self.file_id = file_id
         self.file_external_id = file_external_id
+        self.file_instance_id = file_instance_id
         self.first_page = first_page
         self.last_page = last_page
 
-        if not exactly_one_is_not_none(file_id, file_external_id):
-            raise ValueError("Exactly one of file_id and file_external_id must be set for a file reference")
+        if not exactly_one_is_not_none(file_id, file_external_id, file_instance_id):
+            raise ValueError("File references must have exactly one of file_id, file_external_id and file_instance_id.")
         if exactly_one_is_not_none(first_page, last_page):
             raise ValueError("If the page range feature is used, both first page and last page must be set")
 
-    def to_api_item(self) -> dict[str, str | int | dict[str, int]]:
-        if self.file_id is None and self.file_external_id is not None:
-            item: dict[str, str | int | dict[str, int]] = {"fileExternalId": self.file_external_id}
-        if self.file_id is not None and self.file_external_id is None:
+    def to_api_item(self) -> dict[str, str | int | dict[str, int] | dict[str, str]]:
+        if self.file_id is None and self.file_external_id is not None and self.file_instance_id is None:
+            item: dict[str, str | int | dict[str, int] | dict[str, str]] = {"fileExternalId": self.file_external_id}
+        if self.file_id is not None and self.file_external_id is None and self.file_instance_id is None:
             item = {"fileId": self.file_id}
+        if self.file_id is None and self.file_external_id is None and self.file_instance_id is not None:
+            item = {"fileInstanceId": self.file_instance_id.dump(include_instance_type=False)}
+
         if self.first_page is not None and self.last_page is not None:
             item["pageRange"] = {"begin": self.first_page, "end": self.last_page}
         return item
@@ -460,6 +479,7 @@ class DiagramDetectItem(CogniteResource):
         self,
         file_id: int | None = None,
         file_external_id: str | None = None,
+        file_instance_id: dict[str, str] | None = None,
         annotations: list | None = None,
         error_message: str | None = None,
         cognite_client: CogniteClient | None = None,
@@ -468,6 +488,7 @@ class DiagramDetectItem(CogniteResource):
     ) -> None:
         self.file_id = file_id
         self.file_external_id = file_external_id
+        self.file_instance_id = file_instance_id
         self.annotations = annotations
         self.error_message = error_message
         self._cognite_client = cast("CogniteClient", cognite_client)
@@ -484,7 +505,7 @@ class DiagramDetectItem(CogniteResource):
             pandas.DataFrame: The dataframe.
         """
         df = super().to_pandas(camel_case=camel_case)
-        df.loc["annotations"] = f"{len(df['annotations'])} annotations"
+        df.loc["annotations"] = f"{len(self.annotations or [])} annotations"
         return df
 
 
@@ -532,7 +553,7 @@ class DiagramDetectResults(ContextualizationJob):
 
 
 # Vision dataclasses
-FeatureClass = Union[Type[TextRegion], Type[AssetLink], Type[ObjectDetection]]
+FeatureClass = type[TextRegion] | type[AssetLink] | type[ObjectDetection]
 
 
 class VisionFeature(str, Enum):
@@ -542,6 +563,10 @@ class VisionFeature(str, Enum):
     # The features below are in beta
     INDUSTRIAL_OBJECT_DETECTION = "IndustrialObjectDetection"
     PERSONAL_PROTECTIVE_EQUIPMENT_DETECTION = "PersonalProtectiveEquipmentDetection"
+    DIAL_GAUGE_DETECTION = "DialGaugeDetection"
+    LEVEL_GAUGE_DETECTION = "LevelGaugeDetection"
+    DIGITAL_GAUGE_DETECTION = "DigitalGaugeDetection"
+    VALVE_DETECTION = "ValveDetection"
 
     @classmethod
     def beta_features(cls) -> set[VisionFeature]:
@@ -556,6 +581,60 @@ class VisionExtractPredictions(VisionResource):
     industrial_object_predictions: list[ObjectDetection] | None = None
     people_predictions: list[ObjectDetection] | None = None
     personal_protective_equipment_predictions: list[ObjectDetection] | None = None
+    digital_gauge_predictions: list[ObjectDetection] | None = None
+    dial_gauge_predictions: list[KeypointCollectionWithObjectDetection] | None = None
+    level_gauge_predictions: list[KeypointCollectionWithObjectDetection] | None = None
+    valve_predictions: list[KeypointCollectionWithObjectDetection] | None = None
+
+    @classmethod
+    def _load(cls, resource: dict[str, Any], cognite_client: Any = None) -> VisionExtractPredictions:
+        return cls(
+            text_predictions=[
+                TextRegion._load(text_prediction) for text_prediction in resource.get("textPredictions", [])
+            ]
+            or None,
+            asset_tag_predictions=[
+                AssetLink._load(asset_tag_prediction)
+                for asset_tag_prediction in resource.get("assetTagPredictions", [])
+            ]
+            or None,
+            industrial_object_predictions=[
+                ObjectDetection._load(industrial_object_prediction)
+                for industrial_object_prediction in resource.get("industrialObjectPredictions", [])
+            ]
+            or None,
+            people_predictions=[
+                ObjectDetection._load(people_prediction) for people_prediction in resource.get("peoplePredictions", [])
+            ]
+            or None,
+            personal_protective_equipment_predictions=[
+                ObjectDetection._load(personal_protective_equipment_prediction)
+                for personal_protective_equipment_prediction in resource.get(
+                    "personalProtectiveEquipmentPredictions", []
+                )
+            ]
+            or None,
+            digital_gauge_predictions=[
+                ObjectDetection._load(digital_gauge_prediction)
+                for digital_gauge_prediction in resource.get("digitalGaugePredictions", [])
+            ]
+            or None,
+            dial_gauge_predictions=[
+                KeypointCollectionWithObjectDetection._load(dial_gauge_prediction)
+                for dial_gauge_prediction in resource.get("dialGaugePredictions", [])
+            ]
+            or None,
+            level_gauge_predictions=[
+                KeypointCollectionWithObjectDetection._load(level_gauge_prediction)
+                for level_gauge_prediction in resource.get("levelGaugePredictions", [])
+            ]
+            or None,
+            valve_predictions=[
+                KeypointCollectionWithObjectDetection._load(valve_prediction)
+                for valve_prediction in resource.get("valvePredictions", [])
+            ]
+            or None,
+        )
 
 
 VISION_FEATURE_MAP: dict[str, Any] = {
@@ -564,15 +643,32 @@ VISION_FEATURE_MAP: dict[str, Any] = {
     "industrial_object_predictions": ObjectDetection,
     "people_predictions": ObjectDetection,
     "personal_protective_equipment_predictions": ObjectDetection,
+    "digital_gauge_predictions": ObjectDetection,
+    "dial_gauge_predictions": KeypointCollectionWithObjectDetection,
+    "level_gauge_predictions": KeypointCollectionWithObjectDetection,
+    "valve_predictions": KeypointCollectionWithObjectDetection,
 }
 
 
-VISION_ANNOTATION_TYPE_MAP: dict[str, str] = {
+VISION_ANNOTATION_TYPE_MAP: dict[str, str | dict[str, str]] = {
     "text_predictions": "images.TextRegion",
     "asset_tag_predictions": "images.AssetLink",
     "industrial_object_predictions": "images.ObjectDetection",
     "people_predictions": "images.ObjectDetection",
     "personal_protective_equipment_predictions": "images.ObjectDetection",
+    "digital_gauge_predictions": "images.ObjectDetection",
+    "dial_gauge_predictions": {
+        "keypoint_collection": "images.KeypointCollection",
+        "object_detection": "images.ObjectDetection",
+    },
+    "level_gauge_predictions": {
+        "keypoint_collection": "images.KeypointCollection",
+        "object_detection": "images.ObjectDetection",
+    },
+    "valve_predictions": {
+        "keypoint_collection": "images.KeypointCollection",
+        "object_detection": "images.ObjectDetection",
+    },
 }
 
 
@@ -607,6 +703,32 @@ class PersonalProtectiveEquipmentDetectionParameters(VisionResource, ThresholdPa
     pass
 
 
+@dataclass
+class DialGaugeDetection(VisionResource, ThresholdParameter):
+    min_level: float | None = None
+    max_level: float | None = None
+    dead_angle: float | None = None
+    non_linear_angle: float | None = None
+
+
+@dataclass
+class LevelGaugeDetection(VisionResource, ThresholdParameter):
+    min_level: float | None = None
+    max_level: float | None = None
+
+
+@dataclass
+class DigitalGaugeDetection(VisionResource, ThresholdParameter):
+    comma_position: int | None = None
+    min_num_digits: int | None = None
+    max_num_digits: int | None = None
+
+
+@dataclass
+class ValveDetection(VisionResource, ThresholdParameter):
+    pass
+
+
 class DetectJobBundle:
     _RESOURCE_PATH = "/context/diagram/detect/"
     _STATUS_PATH = "/context/diagram/detect/status"
@@ -697,6 +819,40 @@ class FeatureParameters(VisionResource):
     people_detection_parameters: PeopleDetectionParameters | None = None
     industrial_object_detection_parameters: IndustrialObjectDetectionParameters | None = None
     personal_protective_equipment_detection_parameters: PersonalProtectiveEquipmentDetectionParameters | None = None
+    digital_gauge_detection_parameters: DigitalGaugeDetection | None = None
+    dial_gauge_detection_parameters: DialGaugeDetection | None = None
+    level_gauge_detection_parameters: LevelGaugeDetection | None = None
+    valve_detection_parameters: ValveDetection | None = None
+
+    @classmethod
+    def _load(cls, resource: dict[str, Any], cognite_client: Any = None) -> FeatureParameters:
+        return cls(
+            text_detection_parameters=load_resource(resource, TextDetectionParameters, "textDetectionParameters"),
+            asset_tag_detection_parameters=load_resource(
+                resource, AssetTagDetectionParameters, "assetTagDetectionParameters"
+            ),
+            people_detection_parameters=load_resource(resource, PeopleDetectionParameters, "peopleDetectionParameters"),
+            industrial_object_detection_parameters=load_resource(
+                resource, IndustrialObjectDetectionParameters, "industrialObjectDetectionParameters"
+            ),
+            personal_protective_equipment_detection_parameters=load_resource(
+                resource,
+                PersonalProtectiveEquipmentDetectionParameters,
+                "personalProtectiveEquipmentDetectionParameters",
+            ),
+            digital_gauge_detection_parameters=load_resource(
+                resource, DigitalGaugeDetection, "digitalGaugeDetectionParameters"
+            ),
+            dial_gauge_detection_parameters=load_resource(
+                resource,
+                DialGaugeDetection,
+                "dialGaugeDetectionParameters",
+            ),
+            level_gauge_detection_parameters=load_resource(
+                resource, LevelGaugeDetection, "levelGaugeDetectionParameters"
+            ),
+            valve_detection_parameters=load_resource(resource, ValveDetection, "valveDetectionParameters"),
+        )
 
 
 class VisionJob(ContextualizationJob):
@@ -734,7 +890,7 @@ class VisionExtractItem(CogniteResource):
         self._cognite_client = cast("CogniteClient", cognite_client)
 
     @classmethod
-    def _load(cls, resource: dict | str, cognite_client: CogniteClient | None = None) -> VisionExtractItem:
+    def _load(cls, resource: dict, cognite_client: CogniteClient | None = None) -> VisionExtractItem:
         """Override CogniteResource._load so that we can convert the dicts returned by the API to data classes"""
         extracted_item = super()._load(resource, cognite_client=cognite_client)
         if isinstance(extracted_item.predictions, dict):
@@ -742,7 +898,7 @@ class VisionExtractItem(CogniteResource):
             extracted_item.predictions = cls._process_predictions_dict(extracted_item._predictions_dict)
         return extracted_item
 
-    def dump(self, camel_case: bool = False) -> dict[str, Any]:
+    def dump(self, camel_case: bool = True) -> dict[str, Any]:
         item_dump = super().dump(camel_case=camel_case)
         # Replace the loaded VisionExtractPredictions with its corresponding dict representation
         if "predictions" in item_dump and isinstance(self._predictions_dict, dict):
@@ -811,22 +967,44 @@ class VisionExtractJob(VisionJob):
         creating_app: str | None = None,
         creating_app_version: str | None = None,
     ) -> list[Annotation]:
-        return [
-            Annotation(
-                annotated_resource_id=item.file_id,
-                annotation_type=VISION_ANNOTATION_TYPE_MAP[prediction_type],
-                data=data,
-                annotated_resource_type="file",
-                status="suggested",
-                creating_app=creating_app or "cognite-sdk-python",
-                creating_app_version=creating_app_version or self._cognite_client.version,
-                creating_user=creating_user or None,
-            )
-            for item in self.items or []
-            if item.predictions is not None
-            for prediction_type, prediction_data_list in item.predictions.dump().items()
-            for data in prediction_data_list
-        ]
+        annotations = []
+
+        for item in self.items or []:
+            if item.predictions is not None:
+                for prediction_type, prediction_data_list in item.predictions.dump(camel_case=False).items():
+                    for data in prediction_data_list:
+                        # check if multiple annotations represent the same prediction (e.g. in case of dial gauges, level gauges and valves)
+                        annotation_type = VISION_ANNOTATION_TYPE_MAP[prediction_type]
+                        if isinstance(annotation_type, dict):
+                            for key, value in annotation_type.items():
+                                annotation = Annotation(
+                                    annotated_resource_id=item.file_id,
+                                    annotation_type=value,
+                                    data=data[key],
+                                    annotated_resource_type="file",
+                                    status="suggested",
+                                    creating_app=creating_app or "cognite-sdk-python",
+                                    creating_app_version=creating_app_version or self._cognite_client.version,
+                                    creating_user=creating_user or None,
+                                )
+                                annotations.append(annotation)
+                        elif isinstance(annotation_type, str):
+                            annotation = Annotation(
+                                annotated_resource_id=item.file_id,
+                                annotation_type=annotation_type,
+                                data=data,
+                                annotated_resource_type="file",
+                                status="suggested",
+                                creating_app=creating_app or "cognite-sdk-python",
+                                creating_app_version=creating_app_version or self._cognite_client.version,
+                                creating_user=creating_user or None,
+                            )
+
+                            annotations.append(annotation)
+                        else:
+                            raise TypeError("annotation_type must be str or dict")
+
+        return annotations
 
     def save_predictions(
         self,
@@ -850,9 +1028,8 @@ class VisionExtractJob(VisionJob):
             annotations = self._predictions_to_annotations(
                 creating_user=creating_user, creating_app=creating_app, creating_app_version=creating_app_version
             )
-            if annotations:
-                return self._cognite_client.annotations.suggest(annotations=annotations)
-            raise CogniteException("Extract job does not contain any predictions.")
+
+            return self._cognite_client.annotations.suggest(annotations=annotations if annotations else [])
 
         raise CogniteException(
             "Extract job is not completed. If the job is queued or running, wait for completion and try again"
@@ -870,3 +1047,212 @@ class ResourceReference(CogniteResource):
 
 class ResourceReferenceList(CogniteResourceList[ResourceReference]):
     _RESOURCE = ResourceReference
+
+
+class DirectionWeights(CogniteObject):
+    """Direction weights for the text graph that control how far text boxes can be one from another
+    in a particular direction before they are no longer connected in the same graph. Lower value means
+    larger distance is allowed.
+
+    Args:
+        left (float | None): Weight for the connection towards text boxes to the left.
+        right (float | None): Weight for the connection towards text boxes to the right.
+        up (float | None): Weight for the connection towards text boxes above.
+        down (float | None): Weight for the connection towards text boxes below.
+    """
+
+    def __init__(
+        self,
+        left: float | None = None,
+        right: float | None = None,
+        up: float | None = None,
+        down: float | None = None,
+    ) -> None:
+        self.left = left
+        self.right = right
+        self.up = up
+        self.down = down
+
+    @classmethod
+    def _load(cls, resource: dict[str, Any], cognite_client: CogniteClient | None = None) -> Self:
+        return cls(
+            left=resource.get("left"),
+            right=resource.get("right"),
+            up=resource.get("up"),
+            down=resource.get("down"),
+        )
+
+
+class CustomizeFuzziness(CogniteObject):
+    """Additional requirements for the fuzzy matching algorithm. The fuzzy match is allowed if any of these are true for each match candidate. The overall minFuzzyScore still applies, but a stricter fuzzyScore can be set here, which would not be enforced if either the minChars or maxBoxes conditions are met, making it possible to exclude detections using replacements if they are either short, or combined from many boxes.
+
+    Args:
+        fuzzy_score (float | None): The minimum fuzzy score of the candidate match.
+        max_boxes (int | None): Maximum number of text boxes the potential match is composed of.
+        min_chars (int | None): The minimum number of characters that must be present in the candidate match string.
+    """
+
+    def __init__(
+        self,
+        fuzzy_score: float | None = None,
+        max_boxes: int | None = None,
+        min_chars: int | None = None,
+    ) -> None:
+        self.fuzzy_score = fuzzy_score
+        self.max_boxes = max_boxes
+        self.min_chars = min_chars
+
+    @classmethod
+    def _load(cls, resource: dict[str, Any], cognite_client: CogniteClient | None = None) -> Self:
+        return cls(
+            fuzzy_score=resource.get("fuzzyScore"),
+            max_boxes=resource.get("maxBoxes"),
+            min_chars=resource.get("minChars"),
+        )
+
+
+class ConnectionFlags:
+    """Connection flags for token graph. These are passed as an array of strings to the API. Only flags set to True are included in the array. There is no need to set any flags to False.
+
+    Args:
+        natural_reading_order (bool): Only connect text regions that are in natural reading order (i.e. top to bottom and left to right).
+        no_text_inbetween (bool): Only connect text regions that are not separated by other text regions.
+        **flags (bool): Other flags.
+    """
+
+    def __init__(
+        self,
+        natural_reading_order: bool = False,
+        no_text_inbetween: bool = False,
+        **flags: bool,
+    ) -> None:
+        self._flags = {
+            "natural_reading_order": natural_reading_order,
+            "no_text_inbetween": no_text_inbetween,
+            **flags,
+        }
+
+    def dump(self, camel_case: bool = False) -> list[str]:
+        return [k for k, v in self._flags.items() if v]
+
+    @classmethod
+    def load(cls, resource: list[str], cognite_client: CogniteClient | None = None) -> Self:
+        return cls(**{flag: True for flag in resource})
+
+
+class DiagramDetectConfig(CogniteObject):
+    """`Configuration options for the diagrams/detect endpoint <https://api-docs.cognite.com/20230101-beta/tag/Engineering-diagrams/operation/diagramDetect/#!path=configuration&t=request>`_.
+
+    Args:
+        annotation_extract (bool | None): Read SHX text embedded in the diagram file. If present, this text will override overlapping OCR text. Cannot be used at the same time as read_embedded_text.
+        case_sensitive (bool | None): Case sensitive text matching. Defaults to True.
+        connection_flags (ConnectionFlags | list[str] | None): Connection flags for token graph. Two flags are supported thus far: `no_text_inbetween` and `natural_reading_order`.
+        customize_fuzziness (CustomizeFuzziness | dict[str, Any] | None): Additional requirements for the fuzzy matching algorithm. The fuzzy match is allowed if any of these are true for each match candidate. The overall minFuzzyScore still applies, but a stricter fuzzyScore can be set here, which would not be enforced if either the minChars or maxBoxes conditions are met, making it possible to exclude detections using replacements if they are either short, or combined from many boxes.
+        direction_delta (float | None): Maximum angle between the direction of two text boxes for them to be connected. Directions are currently multiples of 90 degrees.
+        direction_weights (DirectionWeights | dict[str, Any] | None): Direction weights that control how far subsequent ocr text boxes can be from another in a particular direction and still be combined into the same detection. Lower value means larger distance is allowed. The direction is relative to the text orientation.
+        min_fuzzy_score (float | None): For each detection, this controls to which degree characters can be replaced from the OCR text with similar characters, e.g. I and 1. A value of 1 will disable character replacements entirely.
+        read_embedded_text (bool | None): Read text embedded in the PDF file. If present, this text will override overlapping OCR text.
+        remove_leading_zeros (bool | None): Disregard leading zeroes when matching tags (e.g. "A0001" will match "A1")
+        substitutions (dict[str, list[str]] | None): Override the default mapping of characters to an array of allowed substitute characters. The default mapping contains characters commonly confused by OCR. Provide your custom mapping in the format like so: {"0": ["O", "Q"], "1": ["l", "I"]}. This means: 0 (zero) is allowed to be replaced by uppercase letter O or Q, and 1 (one) is allowed to be replaced by lowercase letter l or uppercase letter I. No other replacements are allowed.
+        **params (Any): Other parameters. The parameter name will be converted to camel case but the value will be passed as is.
+
+    Example:
+
+        Configure a call to digrams detect endpoint:
+
+            >>> from cognite.client import CogniteClient
+            >>> from cognite.client.data_classes.contextualization import ConnectionFlags, DiagramDetectConfig
+            >>> client = CogniteClient()
+            >>> config = DiagramDetectConfig(
+            ...     remove_leading_zeros=True,
+            ...     connection_flags=ConnectionFlags(
+            ...         no_text_inbetween=True,
+            ...         natural_reading_order=True,
+            ...     )
+            ... )
+            >>> job = client.diagrams.detect(entities=[{"name": "A1"}], file_id=123, config=config)
+
+        If you want to use an undocumented parameter, you can pass it as a keyword argument:
+
+            >>> config_with_undoducmented_params = DiagramDetectConfig(
+            ...     remove_leading_zeros=True,
+            ...     connection_flags=ConnectionFlags(new_undocumented_flag=True),
+            ...     new_undocumented_param={"can_be_anything": True},
+            ... )
+    """
+
+    def __init__(
+        self,
+        annotation_extract: bool | None = None,
+        case_sensitive: bool | None = None,
+        connection_flags: ConnectionFlags | list[str] | None = None,
+        customize_fuzziness: CustomizeFuzziness | dict[str, Any] | None = None,
+        direction_delta: float | None = None,
+        direction_weights: DirectionWeights | dict[str, Any] | None = None,
+        min_fuzzy_score: float | None = None,
+        read_embedded_text: bool | None = None,
+        remove_leading_zeros: bool | None = None,
+        substitutions: dict[str, list[str]] | None = None,
+        **params: Any,
+    ) -> None:
+        self.annotation_extract = annotation_extract
+        self.case_sensitive = case_sensitive
+        self.connection_flags = connection_flags
+        self.customize_fuzziness = customize_fuzziness
+        self.direction_delta = direction_delta
+        self.direction_weights = direction_weights
+        self.min_fuzzy_score = min_fuzzy_score
+        self.read_embedded_text = read_embedded_text
+        self.remove_leading_zeros = remove_leading_zeros
+        self.substitutions = substitutions
+
+        _known_params = {to_camel_case(k): k for k in vars(self)}
+        self._extra_params = {}
+        for param_name, value in params.items():
+            if known := _known_params.get(to_camel_case(param_name)):
+                raise ValueError(
+                    f"Provided parameter name `{param_name}` collides with a known parameter `{known}`. Please use it insead."
+                )
+            self._extra_params[param_name] = value
+
+    def dump(self, camel_case: bool = True) -> dict[str, Any]:
+        dumped = super().dump(camel_case=camel_case)
+
+        for param_name, v in self._extra_params.items():
+            dumped[to_camel_case(param_name) if camel_case else param_name] = v
+
+        remove_keys: list[str] = []
+        for param_name, v in dumped.items():
+            if isinstance(v, CogniteObject):
+                dumped[param_name] = v.dump(camel_case=camel_case)
+            elif isinstance(v, ConnectionFlags):
+                cf = v.dump()
+                if cf:
+                    dumped[param_name] = cf
+                else:
+                    remove_keys.append(param_name)
+
+        for param_name in remove_keys:
+            del dumped[param_name]
+
+        return dumped
+
+    @classmethod
+    def _load(cls, resource: dict[str, Any], cognite_client: CogniteClient | None = None) -> Self:
+        resource_copy = resource.copy()
+
+        if con_flg := resource_copy.pop("connectionFlags", None):
+            con_flg = ConnectionFlags.load(con_flg)
+        if cus_fuz := resource_copy.pop("customizeFuzziness", None):
+            cus_fuz = CustomizeFuzziness.load(cus_fuz)
+        if dir_wgt := resource_copy.pop("directionWeights", None):
+            dir_wgt = DirectionWeights.load(dir_wgt)
+
+        snake_cased = convert_all_keys_to_snake_case(resource_copy)
+
+        return cls(
+            connection_flags=con_flg,
+            customize_fuzziness=cus_fuz,
+            direction_weights=dir_wgt,
+            **snake_cased,
+        )
