@@ -2,12 +2,15 @@ from __future__ import annotations
 
 import numbers
 from abc import ABC
+from collections.abc import Sequence
+from dataclasses import dataclass
 from typing import (
+    Any,
+    ClassVar,
     Generic,
     Literal,
     NoReturn,
     Protocol,
-    Sequence,
     TypeVar,
     cast,
     overload,
@@ -15,45 +18,104 @@ from typing import (
 
 from cognite.client._constants import MAX_VALID_INTERNAL_ID
 from cognite.client.utils._auxiliary import split_into_chunks
+from cognite.client.utils.useful_types import SequenceNotStr
 
-T_ID = TypeVar("T_ID", int, str)
+
+@dataclass(frozen=True)
+class InstanceId:
+    _instance_type: ClassVar[Literal["node", "edge"]]
+    space: str
+    external_id: str
+
+    def dump(self, camel_case: bool = True, include_instance_type: bool = True) -> dict[str, str]:
+        return {"space": self.space, "externalId" if camel_case else "external_id": self.external_id}
+
+    @classmethod
+    def load(cls: type[T_InstanceId], data: dict) -> T_InstanceId:
+        if isinstance(data, cls):
+            return data
+        if "externalId" in data:
+            return cls(space=data["space"], external_id=data["externalId"])
+        if "external_id" in data:
+            return cls(space=data["space"], external_id=data["external_id"])
+        raise KeyError(f"Cannot load {data} into {cls}, missing 'externalId' or 'external_id' key")
+
+    @property
+    def instance_type(self) -> Literal["node", "edge"]:
+        return self._instance_type
+
+    def as_tuple(self) -> tuple[str, str]:
+        return self.space, self.external_id
+
+    def __repr__(self) -> str:
+        return f"{self.__class__.__name__}(space={self.space!r}, external_id={self.external_id!r})"
+
+
+T_InstanceId = TypeVar("T_InstanceId", bound=InstanceId)
+
+T_ID = TypeVar("T_ID", int, str, InstanceId)
 
 
 class IdentifierCore(Protocol):
-    def as_dict(self, camel_case: bool = True) -> dict:
-        ...
+    def as_dict(self, camel_case: bool = True) -> dict: ...
 
-    def as_primitive(self) -> str | int:
-        ...
+    def as_primitive(self) -> str | int: ...
 
 
 class Identifier(Generic[T_ID]):
     def __init__(self, value: T_ID) -> None:
+        if not isinstance(value, (int, str, InstanceId)):
+            raise TypeError(f"Expected id/external_id to be of type int or str, got {value} of type {type(id)}")
         self.__value: T_ID = value
 
-    @classmethod
-    def of_either(cls, id: int | None, external_id: str | None) -> Identifier:
-        if id is external_id is None:
-            raise ValueError("Exactly one of id or external id must be specified, got neither")
-        elif id is not None:
-            if external_id is not None:
-                raise ValueError("Exactly one of id or external id must be specified, got both")
-            elif not 1 <= id <= MAX_VALID_INTERNAL_ID:
-                raise ValueError(f"Invalid id, must satisfy: 1 <= id <= {MAX_VALID_INTERNAL_ID}")
-        return Identifier(id or external_id)
+    def __eq__(self, other: Any) -> bool:
+        return isinstance(other, Identifier) and self.__value == other.__value
+
+    def __hash__(self) -> int:
+        return hash(self.__value)
 
     @classmethod
-    def load(cls, id: int | None = None, external_id: str | None = None) -> Identifier:
+    def of_either(cls, id: int | None, external_id: str | None, instance_id: InstanceId | None = None) -> Identifier:
+        if id is external_id is instance_id is None:
+            raise ValueError("Exactly one of id, external id, or instance_id must be specified, got neither")
+        elif id is not None:
+            if external_id is not None or instance_id is not None:
+                raise ValueError("Exactly one of id, external id, or instance_id must be specified, got multiple")
+            elif not isinstance(id, int):
+                raise TypeError(f"Invalid id, expected int, got {type(id)}")
+            elif not 1 <= id <= MAX_VALID_INTERNAL_ID:
+                raise ValueError(f"Invalid id, must satisfy: 1 <= id <= {MAX_VALID_INTERNAL_ID}")
+        elif external_id is not None:
+            if instance_id is not None:
+                raise ValueError("Exactly one of id, external id, or instance_id must be specified, got multiple")
+            elif not isinstance(external_id, str):
+                raise TypeError(f"Invalid external_id, expected str, got {type(external_id)}")
+        elif instance_id is not None:
+            if isinstance(instance_id, dict):
+                instance_id = InstanceId.load(instance_id)
+            if not isinstance(instance_id, InstanceId):
+                raise TypeError(f"Invalid instance_id, expected InstanceId, got {type(instance_id)}")
+        return Identifier(id or external_id or instance_id)
+
+    @classmethod
+    def load(
+        cls, id: int | None = None, external_id: str | None = None, instance_id: InstanceId | None = None
+    ) -> Identifier:
         if id is not None:
             return Identifier(id)
         if external_id is not None:
             return Identifier(external_id)
-        raise ValueError("At least one of id and external id must be specified")
+        if instance_id is not None:
+            return Identifier(instance_id)
+        raise ValueError("At least one of id, external id, instance_id must be specified")
 
     def name(self, camel_case: bool = False) -> str:
         if self.is_id:
             return "id"
-        return "externalId" if camel_case else "external_id"
+        elif self.is_instance_id:
+            return "instanceId" if camel_case else "instance_id"
+        else:
+            return "externalId" if camel_case else "external_id"
 
     def as_primitive(self) -> T_ID:
         return self.__value
@@ -66,8 +128,15 @@ class Identifier(Generic[T_ID]):
     def is_external_id(self) -> bool:
         return isinstance(self.__value, str)
 
+    @property
+    def is_instance_id(self) -> bool:
+        return isinstance(self.__value, InstanceId)
+
     def as_dict(self, camel_case: bool = True) -> dict[str, T_ID]:
-        return {self.name(camel_case): self.__value}
+        if isinstance(self.__value, InstanceId):
+            return {"instanceId": self.__value.dump(camel_case=camel_case, include_instance_type=False)}  # type: ignore[dict-item]
+        else:
+            return {self.name(camel_case): self.__value}
 
     def as_tuple(self, camel_case: bool = True) -> tuple[str, T_ID]:
         return self.name(camel_case), self.__value
@@ -87,6 +156,20 @@ class UserIdentifier:
         return self.__value
 
 
+class Username:
+    def __init__(self, value: str) -> None:
+        self.__value: str = value
+
+    def name(self, camel_case: bool = False) -> str:
+        return "username"
+
+    def as_dict(self, camel_case: bool = True) -> dict[str, str]:
+        return {self.name(camel_case): self.__value}
+
+    def as_primitive(self) -> str:
+        return self.__value
+
+
 class WorkflowVersionIdentifier:
     def __init__(self, version: str, workflow_external_id: str) -> None:
         self.__version: str = version
@@ -130,12 +213,10 @@ class DataModelingIdentifier:
         raise AttributeError(f"Not supported for {type(self).__name__} implementation")
 
 
-class ExternalId(Identifier[str]):
-    ...
+class ExternalId(Identifier[str]): ...
 
 
-class InternalId(Identifier[int]):
-    ...
+class InternalId(Identifier[int]): ...
 
 
 T_Identifier = TypeVar("T_Identifier", bound=IdentifierCore)
@@ -143,8 +224,6 @@ T_Identifier = TypeVar("T_Identifier", bound=IdentifierCore)
 
 class IdentifierSequenceCore(Generic[T_Identifier], ABC):
     def __init__(self, identifiers: list[T_Identifier], is_singleton: bool) -> None:
-        if not identifiers:
-            raise ValueError("No identifiers specified")
         self._identifiers = identifiers
         self.__is_singleton = is_singleton
 
@@ -159,7 +238,7 @@ class IdentifierSequenceCore(Generic[T_Identifier], ABC):
 
     def assert_singleton(self) -> None:
         if not self.is_singleton():
-            raise ValueError("Exactly one of id or external id must be specified")
+            raise ValueError("Exactly one of id or external_id must be specified")
 
     def as_singleton(self) -> SingletonIdentifierSequence:
         self.assert_singleton()
@@ -181,7 +260,7 @@ class IdentifierSequenceCore(Generic[T_Identifier], ABC):
         return len(self) == len(set(self.as_primitives()))
 
     @staticmethod
-    def unwrap_identifier(identifier: str | int | dict) -> str | int:
+    def unwrap_identifier(identifier: str | int | dict) -> str | int | InstanceId:
         if isinstance(identifier, (str, int)):
             return identifier
         if "externalId" in identifier:
@@ -190,7 +269,14 @@ class IdentifierSequenceCore(Generic[T_Identifier], ABC):
             return identifier["id"]
         if "space" in identifier:
             return identifier["space"]
-        raise ValueError(f"{identifier} does not contain 'id' or 'externalId' or 'space'")
+        if "instanceId" in identifier:
+            return InstanceId.load(identifier["instanceId"])
+        raise ValueError(f"{identifier} does not contain 'id' or 'externalId', or 'space'")
+
+    @staticmethod
+    def extract_identifiers(dct: dict[str, Any]) -> dict[str, str | int]:
+        """An API payload might look like {"id": 1, "before": "2w-ago", ...}. This function extracts the identifiers"""
+        return {k: dct[k] for k in ("id", "externalId") if k in dct}
 
 
 T_IdentifierSequenceCore = TypeVar("T_IdentifierSequenceCore", bound=IdentifierSequenceCore)
@@ -199,16 +285,14 @@ T_IdentifierSequenceCore = TypeVar("T_IdentifierSequenceCore", bound=IdentifierS
 class IdentifierSequence(IdentifierSequenceCore[Identifier]):
     @overload
     @classmethod
-    def of(cls, *ids: list[int | str]) -> IdentifierSequence:
-        ...
+    def of(cls, *ids: list[int | str | InstanceId]) -> IdentifierSequence: ...
 
     @overload
     @classmethod
-    def of(cls, *ids: int | str) -> IdentifierSequence:
-        ...
+    def of(cls, *ids: int | str | InstanceId) -> IdentifierSequence: ...
 
     @classmethod
-    def of(cls, *ids: int | str | Sequence[int | str]) -> IdentifierSequence:
+    def of(cls, *ids: int | str | InstanceId | Sequence[int | str | InstanceId]) -> IdentifierSequence:
         if len(ids) == 1 and isinstance(ids[0], Sequence) and not isinstance(ids[0], str):
             return cls([Identifier(val) for val in ids[0]], is_singleton=False)
         else:
@@ -218,14 +302,15 @@ class IdentifierSequence(IdentifierSequenceCore[Identifier]):
     def load(
         cls,
         ids: int | Sequence[int] | None = None,
-        external_ids: str | Sequence[str] | None = None,
+        external_ids: str | SequenceNotStr[str] | SequenceNotStr[str] | None = None,
+        instance_ids: InstanceId | Sequence[InstanceId] | None = None,
         *,
         id_name: str = "",
     ) -> IdentifierSequence:
         if id_name and not id_name.endswith("_"):
             id_name += "_"
         value_passed_as_primitive = False
-        all_identifiers: list[int | str] = []
+        all_identifiers: list[int | str | InstanceId] = []
 
         if ids is not None:
             if isinstance(ids, numbers.Integral):
@@ -244,38 +329,62 @@ class IdentifierSequence(IdentifierSequenceCore[Identifier]):
                 all_identifiers.extend([str(extid) for extid in external_ids])
             else:
                 raise TypeError(
-                    f"{id_name}external_ids must be of type str or Sequence[str]. Found {type(external_ids)}"
+                    f"{id_name}external_ids must be of type str or SequenceNotStr[str]. Found {type(external_ids)}"
+                )
+        if instance_ids is not None:
+            if isinstance(instance_ids, InstanceId):
+                value_passed_as_primitive = True
+                all_identifiers.append(instance_ids)
+            elif isinstance(instance_ids, Sequence):
+                all_identifiers.extend([InstanceId.load(instance_id) for instance_id in instance_ids])  # type: ignore[arg-type]
+            else:
+                raise TypeError(
+                    f"{id_name}instance_ids must be of type InstanceId or Sequence[InstanceId]. Found {type(instance_ids)}"
                 )
 
         is_singleton = value_passed_as_primitive and len(all_identifiers) == 1
         return cls(identifiers=[Identifier(val) for val in all_identifiers], is_singleton=is_singleton)
 
 
-class SingletonIdentifierSequence(IdentifierSequenceCore[Identifier]):
-    ...
+class SingletonIdentifierSequence(IdentifierSequenceCore[Identifier]): ...
 
 
-class DataModelingIdentifierSequence(IdentifierSequenceCore[DataModelingIdentifier]):
-    ...
+class DataModelingIdentifierSequence(IdentifierSequenceCore[DataModelingIdentifier]): ...
 
 
 class UserIdentifierSequence(IdentifierSequenceCore[UserIdentifier]):
     # TODO: Inferred type from inherited methods 'as_dicts' and 'as_primitives' wrongly include 'int'
     @classmethod
-    def load(cls, user_identifiers: str | Sequence[str]) -> UserIdentifierSequence:
+    def load(cls, user_identifiers: str | SequenceNotStr[str]) -> UserIdentifierSequence:
         if isinstance(user_identifiers, str):
             return cls([UserIdentifier(user_identifiers)], is_singleton=True)
 
         elif isinstance(user_identifiers, Sequence):
             return cls(list(map(UserIdentifier, map(str, user_identifiers))), is_singleton=False)
 
-        raise TypeError(f"user_identifiers must be of type str or Sequence[str]. Found {type(user_identifiers)}")
+        raise TypeError(f"user_identifiers must be of type str or SequenceNotStr[str]. Found {type(user_identifiers)}")
 
     def assert_singleton(self) -> None:
         if not self.is_singleton():
             raise ValueError("Exactly one user identifier (string) must be specified")
 
 
+class UsernameSequence(IdentifierSequenceCore[Username]):
+    @classmethod
+    def load(cls, usernames: str | SequenceNotStr[str]) -> UsernameSequence:
+        if isinstance(usernames, str):
+            return cls([Username(usernames)], is_singleton=True)
+
+        elif isinstance(usernames, Sequence):
+            return cls([Username(username) for username in usernames], is_singleton=False)
+
+        raise TypeError(f"usernames must be of type str or SequenceNotStr[str]. Found {type(usernames)}")
+
+    def assert_singleton(self) -> None:
+        if not self.is_singleton():
+            raise ValueError("Exactly one username (string) must be specified")
+
+
 class WorkflowVersionIdentifierSequence(IdentifierSequenceCore[WorkflowVersionIdentifier]):
     @classmethod
     def load(cls, workflow_ids: Sequence[dict]) -> WorkflowVersionIdentifierSequence:
@@ -305,4 +414,4 @@ class WorkflowVersionIdentifierSequence(IdentifierSequenceCore[WorkflowVersionId
             return identifier
         if "workflowExternalId" in identifier and "version" in identifier:
             return identifier["workflowExternalId"], identifier["version"]
-        raise ValueError(f"{identifier} does not contain both 'workflowExternalId' and 'version''")
+        raise ValueError(f"{identifier} does not contain both 'workflowExternalId' and 'version'")
